﻿using Hotline.Caching.Interfaces;
using Hotline.CallCenter.Tels;
using Hotline.Users;
using XF.Domain.Cache;
using XF.Domain.Dependency;
using XF.Domain.Exceptions;

namespace Hotline.Application.Tels;

public class TelApplication : ITelApplication, IScopeDependency
{
    private readonly IUserCacheManager _userCacheManager;
    private readonly ITelRestRepository _telRestRepository;
    private readonly ITypedCache<Work> _cacheWork;
    private readonly IWorkRepository _workRepository;

    public TelApplication(
        IUserCacheManager userCacheManager,
        ITelRestRepository telRestRepository,
        ITypedCache<Work> cacheWork,
        IWorkRepository workRepository)
    {
        _userCacheManager = userCacheManager;
        _telRestRepository = telRestRepository;
        _cacheWork = cacheWork;
        _workRepository = workRepository;
    }

    /// <summary>
    /// 用户签出分机
    /// </summary>
    public async Task SignOutAsync(string userId, CancellationToken cancellationToken)
    {
        var work = _userCacheManager.GetWorkByUser(userId);
        if (work is null) return;

        var telRest = await _telRestRepository.GetAsync(x => x.TelNo == work.TelNo && !x.EndTime.HasValue, cancellationToken);
        if (telRest is not null)
        {
            telRest.EndRest();
            await _telRestRepository.UpdateAsync(telRest, cancellationToken);
        }

        work.OffDuty();
        await _workRepository.UpdateAsync(work, cancellationToken);
        _cacheWork.Remove(work.GetKey(KeyMode.UserId));
        _cacheWork.Remove(work.GetKey(KeyMode.TelNo));
    }

    public async Task SignOutByTelNoAsync(string telNo,CancellationToken cancellationToken)
    {
        var work = _userCacheManager.GetWorkByTelNoExp(telNo);
        if (work is null) return;

        var telRest = await _telRestRepository.GetAsync(x => x.TelNo == work.TelNo && !x.EndTime.HasValue, cancellationToken);
        if (telRest is not null)
        {
            telRest.EndRest();
            await _telRestRepository.UpdateAsync(telRest, cancellationToken);
        }

        work.OffDuty();
        await _workRepository.UpdateAsync(work, cancellationToken);
        _cacheWork.Remove(work.GetKey(KeyMode.UserId));
        _cacheWork.Remove(work.GetKey(KeyMode.TelNo));
    }
}