﻿using Hotline.Share.Enums.FlowEngine;
using Hotline.Share.Enums.Order;

namespace Hotline.Share.Requests
{
    public class DepartmentalProcessingStatisticsDto
    {
        public string Id { get; set; }

        public string OrgCode { get; set; }

        public string ActualHandleOrgCode { get; set; }

        public EOrderStatus? Status { get; set; }

        public DateTime? ExpiredTime { get; set; }

        public DateTime? ActualHandleTime { get; set; }

        public ECounterSignType? CounterSignType { get; set; }

        public bool PublishState { get; set; }
    }

    public class DepartmentalProcessingStatisticsDataDto
    {
        /// <summary>
        /// 部门Code
        /// </summary>
        public string OrgCode { get; set; }

        /// <summary>
        /// 部门名称
        /// </summary>
        public string OrgName { get; set; }

        /// <summary>
        /// 部门类型
        /// </summary>
        public string OrgType { get; set; }

        /// <summary>
        /// 信件总量
        /// </summary>
        public int OrderCountNum { get; set; }

        /// <summary>
        /// 信件已办总量
        /// </summary>
        public int YBOrderCountNum { get; set; }

        /// <summary>
        /// 信件未办总量
        /// </summary>
        public int ZBOrderCountNum { get; set; }

        /// <summary>
        /// 办结率
        /// </summary>
        public double OrderCompletionRate => CalcProportionRate();


        /// <summary>
        /// 已归档
        /// </summary>
        public int Archived { get; set; }

        /// <summary>
        /// 待归档
        /// </summary>
       public int ToBeArchived { get; set; }

        /// <summary>
        /// 待发布  --已归档的就是待发布
        /// </summary>
        public int WaitPublished { get; set; }

        /// <summary>
        /// 已发布公开
        /// </summary>
        public int PublishedOpen { get; set; }

        /// <summary>
        /// 已发布不公开
        /// </summary>
        public int PublishedNoOpen { get; set; }

        /// <summary>
        /// 已办超期
        /// </summary>
        public int YBOverdue { get; set; }

        /// <summary>
        /// 待办超期
        /// </summary>
        public int ZBOverdue { get; set; }

        /// <summary>
        /// 会签已办超期
        /// </summary>
        public int HQYBOverdue { get; set; }

        /// <summary>
        /// 会签待办超期
        /// </summary>
        public int HQZBOverdue { get; set; }

        /// <summary>
        /// 超期件数
        /// </summary>
        public int SubtotalOverdue => YBOverdue + ZBOverdue + HQYBOverdue + HQZBOverdue;

        /// <summary>
        /// 会签超期率
        /// </summary>
        public double HQOverdueRate => CalcOverdueRate();

        /// <summary>
        /// 会签已办
        /// </summary>
        public int DelayEnd { get; set; }

        /// <summary>
        /// 会签待办
        /// </summary>
        public int DelayWait { get; set; }










        /// <summary>
        /// 工单计算办结率
        /// </summary>
        /// <returns></returns>
        public double CalcProportionRate()
        {
            if (OrderCountNum != 0 && YBOrderCountNum != 0)
            {
                return Math.Round((YBOrderCountNum / (double)OrderCountNum) * 100, 2);
            }
            return 0;
        }

        /// <summary>
        /// 计算超期率
        /// </summary>
        /// <returns></returns>
        public double CalcOverdueRate()
        {
            if (OrderCountNum != 0 && SubtotalOverdue != 0)
            {
                return Math.Round((SubtotalOverdue / (double)OrderCountNum) * 100, 2);
            }
            return 0;
        }

    }
}
